Package: santoku
Type: Package
Title: A Versatile Cutting Tool
Version: 1.2.0
Authors@R: 
    c(
    person(given = "David",
           family = "Hugh-Jones",
           role = c("aut", "cre"),
           email = "davidhughjones@gmail.com"),
    person(given = "Daniel",
           family = "Possenriede",
           role = c("ctb"),
           email = "possenriede@gmail.com")
    )
Maintainer: David Hugh-Jones <davidhughjones@gmail.com>
Description: A tool for cutting data into intervals. Allows singleton intervals.
  Always includes the whole range of data by default. Flexible labelling. 
  Convenience functions for cutting by quantiles etc. Handles dates, times, units
  and other vectors.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: bench, bit64, covr, haven, Hmisc, hms, knitr, lubridate,
        purrr, rmarkdown, scales, stringi, testthat (>= 3.2.0), units,
        withr, xts, zoo
Config/testthat/edition: 3
LinkingTo: Rcpp
Depends: R (>= 3.5.0)
Imports: Rcpp, assertthat, glue, lifecycle, rlang, vctrs
URL: https://github.com/hughjonesd/santoku,
        https://hughjonesd.github.io/santoku/
BugReports: https://github.com/hughjonesd/santoku/issues
VignetteBuilder: knitr
RdMacros: lifecycle
NeedsCompilation: yes
Packaged: 2026-04-28 14:15:23 UTC; davidhugh-jones
Author: David Hugh-Jones [aut, cre],
  Daniel Possenriede [ctb]
Repository: CRAN
Date/Publication: 2026-04-28 14:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-01 20:57:12 UTC; unix
Archs: santoku.so.dSYM
