# Generated by roxygen2: do not edit by hand

S3method(apply_format,"function")
S3method(apply_format,character)
S3method(apply_format,list)
S3method(apply_format.character,character)
S3method(apply_format.character,default)
S3method(apply_format.character,numeric)
S3method(apply_format.list,default)
S3method(as.double,breaks)
S3method(brk_width,Duration)
S3method(brk_width,default)
S3method(class_bounds,Date)
S3method(class_bounds,POSIXct)
S3method(class_bounds,default)
S3method(class_bounds,difftime)
S3method(class_bounds,integer64)
S3method(class_bounds,numeric)
S3method(class_bounds,units)
S3method(class_bounds,zoo)
S3method(endpoint_labels,Date)
S3method(endpoint_labels,POSIXt)
S3method(endpoint_labels,default)
S3method(endpoint_labels,double)
S3method(endpoint_labels,integer)
S3method(endpoint_labels,numeric)
S3method(endpoint_labels,quantileBreaks)
S3method(endpoint_labels,sdBreaks)
S3method(format,breaks)
S3method(print,breaks)
S3method(santoku_cast_common,Date)
S3method(santoku_cast_common,Date.Date)
S3method(santoku_cast_common,Date.POSIXct)
S3method(santoku_cast_common,POSIXct)
S3method(santoku_cast_common,POSIXct.Date)
S3method(santoku_cast_common,POSIXct.POSIXct)
S3method(santoku_cast_common,default)
S3method(santoku_cast_common,default.default)
S3method(santoku_cast_common,default.hexmode)
S3method(santoku_cast_common,default.integer64)
S3method(santoku_cast_common,default.octmode)
S3method(santoku_cast_common,default.ts)
S3method(santoku_cast_common,default.zoo)
S3method(santoku_cast_common,double)
S3method(santoku_cast_common,double.default)
S3method(santoku_cast_common,double.integer64)
S3method(santoku_cast_common,hexmode)
S3method(santoku_cast_common,hexmode.default)
S3method(santoku_cast_common,hexmode.hexmode)
S3method(santoku_cast_common,integer64)
S3method(santoku_cast_common,integer64.default)
S3method(santoku_cast_common,integer64.double)
S3method(santoku_cast_common,integer64.integer64)
S3method(santoku_cast_common,octmode)
S3method(santoku_cast_common,octmode.default)
S3method(santoku_cast_common,octmode.octmode)
S3method(santoku_cast_common,ts)
S3method(santoku_cast_common,ts.default)
S3method(santoku_cast_common,zoo)
S3method(santoku_cast_common,zoo.default)
S3method(scaled_endpoints,breaks)
S3method(scaled_endpoints,default)
S3method(sequence_width,Period)
S3method(sequence_width,default)
export(brk_default)
export(brk_equally)
export(brk_evenly)
export(brk_fn)
export(brk_manual)
export(brk_mean_sd)
export(brk_n)
export(brk_pretty)
export(brk_proportions)
export(brk_quantiles)
export(brk_spikes)
export(brk_width)
export(chop)
export(chop_deciles)
export(chop_equally)
export(chop_evenly)
export(chop_fn)
export(chop_mean_sd)
export(chop_n)
export(chop_pretty)
export(chop_proportions)
export(chop_quantiles)
export(chop_spikes)
export(chop_width)
export(dissect)
export(exactly)
export(fillet)
export(is.breaks)
export(kiru)
export(lbl_dash)
export(lbl_date)
export(lbl_datetime)
export(lbl_discrete)
export(lbl_endpoint)
export(lbl_endpoints)
export(lbl_glue)
export(lbl_intervals)
export(lbl_manual)
export(lbl_midpoints)
export(lbl_seq)
export(percent)
export(santoku_cast_common.Date)
export(santoku_cast_common.POSIXct)
export(santoku_cast_common.default)
export(santoku_cast_common.double)
export(santoku_cast_common.hexmode)
export(santoku_cast_common.integer64)
export(santoku_cast_common.octmode)
export(santoku_cast_common.ts)
export(santoku_cast_common.zoo)
export(tab)
export(tab_deciles)
export(tab_dissect)
export(tab_equally)
export(tab_evenly)
export(tab_fn)
export(tab_mean_sd)
export(tab_n)
export(tab_pretty)
export(tab_proportions)
export(tab_quantiles)
export(tab_spikes)
export(tab_width)
import(assertthat)
importFrom(Rcpp,sourceCpp)
importFrom(lifecycle,deprecated)
useDynLib(santoku, .registration = TRUE)
