# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

categorize_impl <- function(x, breaks, left) {
    .Call(`_santoku_categorize_impl`, x, breaks, left)
}

